@@ -26,7 +26,7 @@
 #'@author
 #'Atesh Koul, C'MON unit, Istituto Italiano di Tecnologia
 #'
-#'\email{atesh.koul@@iit.it}
+#'\email{atesh.koul@@gmail.com}
 #'
 #'@references 
 #'
@@ -39,6 +39,19 @@
 #'
 #' @export
 fscore <- function(Data,classCol,featureCol,silent=FALSE){
+  
+  # Make the outcome factor anyways
+  # Modified so that it works even with tibble; force the tibble to be a dataframe
+  # This is not the best way to proceed; Ideally, all the code should be updated 
+  # to work with tibble
+  # make it a bit generic to handle matrices as well along with a warning 
+  permittedDataClass <- c("tbl_df","matrix")
+  if(any(permittedDataClass %in% class(Data))){
+    warning(cat("the data entered is of the class ",class(Data),". Coersing it to be a dataframe. Check results"))
+    Data <- as.data.frame(Data)
+  }
+  
+  
   if(!silent) cat("\nPerforming Feature selection f-score analysis \n\n")
   tr <- function(m) return(sum(diag(m)))
   # separate positive and negative feature sets
